function [pts,costs,tmpSegs] = sampleWormBodyPts(segs,genmodel,SAMPLE_NO)
%function [pts,cost] = sampleWormBodyPts(segs,genmodel)
%Assumes segs.resp is probability, already appropriately scaled

numTypes = length(genmodel.pre);
msgs = cell(numTypes,1);
cpts = cell(numTypes,1);
SCALE = 2; %Default

%Add the reverse direction
tmp = segs;
tmp.u = -tmp.u;
tmp.v = -tmp.v;
segs = appendSegs(segs,tmp);

len = segs.len(1);
wid = segs.w(1);
%Calculate the valid dynamic links
[relX, relY] = relAllPos(segs,segs,-1);
relY = relY - len;
relT = (1 - [segs.u segs.v]*[segs.u segs.v]');
xVar = (wid/3).^2;
yVar = (len/3).^2;
pairPot = relX.^2/xVar + relY.^2/yVar + 0.75*relT/.1;
pairPot = exp(-pairPot'); %pairPot is numc X nump
num = length(segs.resp);
nump = num;
numc = num;

%Go through in reverse order
for segType = genmodel.post,
  %Initialize the unary potential to the appearance
  unPot = segs.resp; unPot = unPot./sum(unPot);
  for kids = find(genmodel.dag(segType,:)),
    unPot = unPot .* msgs{kids}';
  end

  %Add in pairwise potential if we're not a the root
  if segType > 1,
    tmp = pairPot.*repmat(unPot,1,nump);
    msgs{segType} = sum(tmp,1);
    cpts{segType} = tmp./repmat(msgs{segType} + eps,numc,1);
    msgs{segType} = msgs{segType}./(sum(msgs{segType}) + eps);
  else
    msgs{segType} = sum(unPot);
    cpts{segType} = unPot./msgs{segType};
  end
end

if msgs{1} < eps,
  pts = [];
  costs = [];
  return;
end

%Store the indices of segTypes
segInds = cell(numTypes,1);
segInds(:) = deal({[1:num]});

%Debug; Compute tmpSegs, the best assembly found
cptInds = zeros(numTypes,1);
segii = zeros(numTypes,1);
segType = genmodel.root;
[cost, cptInds(segType)] = max(cpts{segType});
segii(segType) = segInds{segType}(cptInds(segType));
for segType = genmodel.pre(2:end),
  parInd = cptInds(genmodel.parents{segType});
  [dummy, cptInds(segType)] = max(cpts{segType}(:,parInd));
  segii(segType) = segInds{segType}(cptInds(segType));
end
tmpSegs = subSegs(segs,segii(genmodel.pre));
tmpSegs.type = genmodel.pre';

%Seed our random number generator
%SAMPLE_NO = 1000;
%SAMPLE_NO = 60;
rand('state',sum(100*clock));
finalInds = rand(SAMPLE_NO,numTypes);
%pts is (XY) x (top-bottom)
pts = zeros(2*2,numTypes,SAMPLE_NO);
costs = zeros(SAMPLE_NO,1);


segPos = [segs.x-segs.u.*segs.len segs.y-segs.v.*segs.len segs.x+segs.u.*segs.len segs.y+segs.v.*segs.len]';

%Convert the resps back to log probabilities, to score the samples
segs.resp = -log(segs.resp);
pairPot = -log(pairPot);

[dummy,I] = sort(cpts{genmodel.root}, 'descend');
for sample_i = 1:SAMPLE_NO
  inds = zeros(numTypes,1);
  %Walk back down the tree
  segType = genmodel.root;
  tmp = cumsum(cpts{segType}); tmp = tmp/tmp(end);
  %inds(segType) = fastGT(tmp,finalInds(sample_i,segType));
  %Store the actual point
  %segi = segInds{segType}(inds(segType));
  inds(segType) = I(round(SAMPLE_NO/2*rand(1))+1);
  segi = inds(segType);
  pts(:,segType,sample_i) = segPos(:,segi);
  costs(sample_i) = segs.resp(segi);
  for segType = genmodel.pre(2:end),
    parInd = inds(genmodel.parents{segType});
    tmp = cumsum(cpts{segType}(:,parInd)); tmp = tmp/tmp(end);
    inds(segType) = fastGT(tmp,finalInds(sample_i,segType));
    segi = segInds{segType}(inds(segType));
    pts(:,segType,sample_i) = segPos(:,segi);
    costs(sample_i) = costs(sample_i) + segs.resp(segi) + pairPot(segi,parInd);
  end
end

%Reshape into standard format
pts = reshape(pts(:,genmodel.pre,:),2*2*length(genmodel.pre),SAMPLE_NO);
