function d = sq_distance_ft(a,b)
% SQ_DISTANCE_FT - computes squared Euclidean distance matrix
%
% E = sq_distance(A,B)
%
%    A - (DXM) matrix 
%    B - (DXN) matrix
% Returns:
%    E - (MxN) Euclidean distances between vectors in A and B
%
%
% Description : 
%    This fully vectorized (VERY FAST!) m-file computes the 
%    Euclidean distance between two vectors by:
%
%                 ||A-B||^2 = ||A||^2 + ||B||^2 - 2*A.B
%
% Note is accepts vectors in column major form to speed stuff up
% B defaults to A

if (nargin < 2)
  b = a;
end

a = double(a);
b = double(b);
m = size(a,2);
n = size(b,2);

if (size(a,1) ~= size(b,1))
   error('A and B should be of same dimensionality');
end

if (m == 1)
  a = [a; zeros(1,m)];
  b = [b; zeros(1,n)];
end

aa = full(sum(a.*a)); bb = full(sum(b.*b)); ab = full(a'*b);
d = repmat(aa',1,n) + repmat(bb,m,1) - 2*ab;

