%cmuTrack.m
global S;
S.firstFrame = 0;
S.lastFrame = 49;
S.formatS = 'jpg';
S.imageS = '0000';
S.imageLocS = 'D:\modeling\images';
S.scaleIm = 1/3;
saveS = 'D:\modeling\Results\result.mat';

MAX_NUM = 1000;
THRESH = .1;
dbclear if warning;

%Define the 1st chain worm model
numTypes1  = 10;
genmodel1.pre = 1:numTypes1;
genmodel1.post = fliplr(genmodel1.pre);
genmodel1.root = 1;
genmodel1.dag = zeros(numTypes1);
genmodel1.parents = cell(numTypes1,1);
for i = 1:numTypes1 - 1,
genmodel1.dag(i,i+1) = 1;
genmodel1.parents{i+1} = i;
end

%Define the 2nd chain worm model
numTypes2  = 8;
genmodel2.pre = 1:numTypes2;
genmodel2.post = fliplr(genmodel2.pre);
genmodel2.root = 1;
genmodel2.dag = zeros(numTypes2);
genmodel2.parents = cell(numTypes2,1);
for i = 1:numTypes2 - 1,
genmodel2.dag(i,i+1) = 1;
genmodel2.parents{i+1} = i;
end

frames = 1:(S.lastFrame - S.firstFrame + 1);
%frames = 1:20; %Debug; just do first 20
[ptsAll1,costsAll1] = deal(cell(length(frames),1));
[ptsAll2,costsAll2] = deal(cell(length(frames),1));
%costsTemp = costsAll;
finalSegs1 = [];
finalSegs2 = [];
%len = 16;
%wid = 8;
len = 8;
wid = 4;
MAX_NUM = 1000;
P_MD = exp(-4); %Probality of a missed detection (not finding a link)

%******MAIN MATCHING PROCEDURE*******
for i = 1:length(frames),
  i
  %im = imclose(imresize(rgb2gray(myImRead(S,frames(i))),S.scaleIm) < 150,strel('disk',3));
  ori_im = imresize(rgb2gray(myImRead(S,frames(i))),S.scaleIm) < 150;
  im = bwmorph(imresize(rgb2gray(myImRead(S,frames(i))),S.scaleIm) < 150,'close');
  temp_im = imresize(rgb2gray(myImRead(S,frames(i))),S.scaleIm) < 30;
  
  %Find a whole bunch of candidate worm links
  res = findCandidateLinks(im,len,wid,MAX_NUM,P_MD);
  
  %This function returns a bunch of worm bodies, sampled from the posterior
  [ptsAll1{i},costsAll1{i},segs1] = sampleWormBodyPts(res,genmodel1);
  [ptsAll2{i},costsAll2{i},segs2] = sampleWormBodyPts(res,genmodel2);

  %Re-evaluate the cost of the samples with global cost measure
  costsAll1{i} = scoreBodyPts(im,ptsAll1{i});
  costsAll2{i} = scoreBodyPts(im,ptsAll2{i});
  %costsTemp{i} = scoreBodyPts(temp_im,ptsAll{i});
  costsTemp{i} = scoreBrightness(im,ori_im,ptsAll1{i});

  segs1.fr = repmat(frames(i),size(segs1.x));
  segs2.fr = repmat(frames(i),size(segs2.x));
  finalSegs1 = appendSegs(finalSegs1,segs1);
  finalSegs2 = appendSegs(finalSegs2,segs2);
  %Show the single best worm body
  %showsegIm(im,segs);
  %drawnow;
  %test{i} = scoreAllPts(im,ptsAll1{i},ptsAll2{i});
end
%Save our results
save(saveS,'ptsAll1','ptsAll2','costsAll1','costsAll2','finalSegs1','finalSegs2','-append');


%To speed stuff up, only keep the best 1/3 of the samples from each frame
NUM = size(ptsAll1{1},2)/1;
for i = 1:length(frames),
    [dummy,I] = sort(costsAll1{i});
    I = I(1:NUM);
    ptsAll1{i} = ptsAll1{i}(:,I);
    costsAll1{i} = costsAll1{i}(I);
    costsTemp{i} = costsTemp{i}(I);
end

NUM = size(ptsAll2{1},2)/1;
for i = 1:length(frames),
    [dummy,I] = sort(costsAll2{i});
    I = I(1:NUM);
    ptsAll2{i} = ptsAll2{i}(:,I);
    costsAll2{i} = costsAll2{i}(I);
end

%Now do dynamic programming to find a sequence of worm bodies
%that moves smoothly over time
ptr = repmat(uint16(0),NUM,length(frames));
cont_W = 5e-2;
numFr = length(frames);

%The first worm
i = 1;
for i = 2:numFr,
    i
    [val,I] = min(sq_distance_ft(ptsAll1{i-1},ptsAll1{i}).*cont_W*2 + repmat(costsAll1{i-1},1,NUM) + repmat(costsTemp{i-1},1,NUM));
    costsAll1{i} = costsAll1{i} + val' + costsTemp{i};
    ptr(:,i) = I';
end

%Back track
Iall1 = zeros(numFr,1);
[dummy,Iall1(end)] = min(costsAll1{numFr});
for i = numFr-1:-1:1,
    Iall1(i) = ptr(Iall1(i+1),i+1);
end

%Now, we have the best worm track
%To make it pretty, we'll smooth it over time by averaging neighboring
%frames
outSegs1 = [];
for i = 1:numFr,
    ptCurr = ptsAll1{i}(:,Iall1(i));
    if i > 1,
        ptPrev = ptsAll1{i-1}(:,Iall1(i-1));
    else
        ptPrev = ptCurr;
    end
    if i < numFr,
        ptNext = ptsAll1{i+1}(:,Iall1(i+1));
    else
        ptNext = ptCurr;
    end
    segs = pt2seg(ptPrev/3 + ptNext/3 + ptCurr/3);
    segs.fr = repmat(i,size(segs.x));
    segs.type = [1:numTypes1]';
    outSegs1 = appendSegs(outSegs1,segs);
    ovlpcost(:,i) = scoreSampleOverlap(ptsAll1{i},ptsAll2{i},Iall1(i));
end

%The second worm
i = 1;
costsAll2{1} = costsAll2{1} + ovlpcost(:,1);
for i = 2:numFr,
    i    
    [val,I] = min(sq_distance_ft(ptsAll2{i-1},ptsAll2{i}).*cont_W + repmat(costsAll2{i-1},1,NUM));
    costsAll2{i} = costsAll2{i} + val' + ovlpcost(:,i);
    ptr(:,i) = I';
end

%Back track
Iall2 = zeros(numFr,1);
[dummy,Iall2(end)] = min(costsAll2{numFr});
for i = numFr-1:-1:1,
    Iall2(i) = ptr(Iall2(i+1),i+1);
end

outSegs2 = [];
for i = 1:numFr,
    ptCurr = ptsAll2{i}(:,Iall2(i));
    if i > 1,
        ptPrev = ptsAll2{i-1}(:,Iall2(i-1));
    else
        ptPrev = ptCurr;
    end
    if i < numFr,
        ptNext = ptsAll2{i+1}(:,Iall2(i+1));
    else
        ptNext = ptCurr;
    end
    segs = pt2seg(ptPrev/3 + ptNext/3 + ptCurr/3);
    segs.fr = repmat(i,size(segs.x));
    segs.type = [1:numTypes2]';
    outSegs2 = appendSegs(outSegs2,segs);   
end

%***DONE WITH ALGORITHM****
%The remaining stuff just makes fancy movies

%Visualize the sampled worm bodies
%clear SM;
%im = imresize(myImRead(S,1),S.scaleIm);
%for i = 1:numFr,
%    ptCurr = ptsAll{i}(:,Iall1(i));
%    %Find all the samples close to the best one
%    inds = find(sum((repmat(ptCurr,1,NUM) - ptsAll{i}).^2) < 2e3);
%    %Draw them all
%    SM(i) = im2frame(showWormBodyPts(size(im),ptsAll{i}(:,inds)));
%    drawnow;
%end

%Low-pass filter the images over time
%tmp = SM;
%for i = 2:length(SM)-1
%    SM(i).cdata = imlincomb(1/3,tmp(i-1).cdata,1/3,tmp(i).cdata,1/3,tmp(i+1).cdata);
%end

iptsetpref('ImshowBorder','tight');
for i = 1:numFr,
 im = imresize(myImRead(S,i),S.scaleIm);
 outSegs = [subSegs(outSegs1,outSegs1.fr == i )
            subSegs(outSegs2,outSegs2.fr == i )];
 showsegImAll(im,outSegs);
 M(i) = getframe;
end

%Two movies are now M (the best track) and SM (the samples)
